unit CHMnet;

interface

uses
   Classes;

// To make Delphi 7.0 happy...
{$IFDEF VER150}
   {$WARN UNSAFE_CODE OFF}
{$ENDIF}

type TCHMlicenseHandle = Pointer;
type TCHMlicenseHandlePtr = ^TCHMlicenseHandle;
type TCHMclientHandle = Pointer;
type TCHMclientHandlePtr = ^TCHMclientHandle;
type TCHMconnectionHandle = Pointer;
type TCHMconnectionHandlePtr = ^TCHMconnectionHandle;
type TCHMlistenerHandle = Pointer;
type TCHMlistenerHandlePtr = ^TCHMlistenerHandle;
type TCHMchameleonEncodingHandle = Pointer;
type TCHMchameleonEncodingHandlePtr = ^TCHMchameleonEncodingHandle;


{: 
  This class can be used by customers to automate the registration of Chameleon 
  run-time instances. This class allows you to programmatically register Chameleon 
  on a user's machine without any user input.
  
  Beginning in version 4.5, this class allows you to register Chameleon without 
  the machine being registered having a direct connection to the Internet. This is 
  useful for companies that want to automate the registration code setup for 
  Chameleon. To make it easy for companies to incorporate Chameleon into their 
  existing licensing process, this class allows you to:
  
   (1) Generate a unique Machine ID for the host machine.
  
   (2) Request a non-expiring license for the Machine ID using the user name and 
  password for the host members account. The automated registration process 
  involves communication with an iNTERFACEWARE server via HTTP. This can be 
  accomplished either from a centrally located computer, or from the actual host 
  machine.
  
   (3) Apply the registration code to the host machine. 
  
  This process makes it easy for companies to get the machine IDs centrally using 
  their own licensing, meaning they can add the extra Chameleon information to 
  their own license file.
  
  Please see http://www.interfaceware.com/manual/api_registration.html for 
  details.
  
}
type TCHMlicense = class(TComponent)
   private
      {$IFNDEF HIDE}
      FHandle : TCHMlicenseHandle;
      
      {: 
        Returns true if the machine has a valid Chameleon license.
        
        This method checks the license status of the target machine but does not send a 
        request to the iNTERFACEWARE server. It returns true if the machine has a valid 
        Chameleon license and false otherwise.
        
      }
      function GetIsLicensed : Boolean;
      
      {: 
        When a Web proxy is required, this property specifies the IP address or hostname 
        for the Web proxy to use.
        
        A Web proxy is required when the local machine does not have direct access to 
        the Internet and needs to forward the registration requests through another 
        server. When it is required, use this property to specify the proxy's IP address 
        or hostname.
        
        A Web proxy server is a server that handles the HTTP requests of its clients by 
        forwarding its requests to other servers (for example, the iNTERFACEWARE 
        server). A client will connect to the Web proxy server to request a file. The 
        proxy server then connects to the specified server and requests the file on 
        behalf of the client.
        
        If your network requires your Internet connection to be set up via a Web proxy, 
        you must specify the proxy your network uses to forward HTTP requests to other 
        servers.
        
        By default, no proxy will be used. To unset a Web proxy, simply set this 
        property to an empty string.
        
        See also the SetProxy method.
        
      }
      function GetProxy : String;
      
      {: 
        Set method for proxy.
        
        See Proxy.
        
      }
      procedure SetProxy(Value : String);
      
      {: 
        This method returns the unique machine ID of the machine to register.
        
      }
      function GetMachineId : String;
      {$ENDIF}
   public
      constructor Create(Owner : TComponent); override;
      destructor Destroy; override;
      
      {: 
        Returns the non-expiring runtime registration code for the unique machine ID 
        from iNTERFACEWARE's server.
        
        This method uses your Internet connection to send an HTTP request on port 80 to 
        the iNTERFACEWARE server. The HTTP request consists of your email address and 
        password (which you use to log into your member's account) and a machine ID 
        uniquely identifying the machine to register. If the request is successful, the 
        iNTERFACEWARE server returns the non-expiring runtime registration code for this 
        unique machine ID. The Description of the registration code will be used to 
        identify it and determine the nature of the license.
        
      }
      function FetchRegistrationCode(MachineId : String; EmailAddress : String; Password : String; Description : String) : String;
      
      {: 
        This method registers the machine using the given registration code.
        
        This method checks if the registration code is valid. If the registration code 
        is invalid, the method throws an exception and does not send a request to the 
        iNTERFACEWARE server. If the registration code is valid, this method registers 
        your machine.
        
        This method only supports non-expiring keys.
        
      }
      procedure RegisterHost(RegistrationCode : String);
      
      {: 
        Returns the expiring trial registration code for the unique machine ID from 
        iNTERFACEWARE's server.
        
        This method uses your Internet connection to send an HTTP request on port 80 to 
        the iNTERFACEWARE server. The HTTP request consists of your email address and 
        password (which you use to log into your member's account) and a machine ID 
        uniquely identifying the machine to register. If the request is successful, the 
        iNTERFACEWARE server returns the expiring trial registration code for this 
        unique machine ID. The Description of the registration code will be used to 
        identify it and determine the nature of the license.
        
      }
      function FetchTrialRegistrationCode(var ExpiryDate : TDateTime; MachineId : String; EmailAddress : String; Password : String; Description : String) : String;
      
      {: 
        Registers the machine using the given expiring trial registration code and 
        expiry date.
        
        This method checks if the registration code is valid. If the registration code 
        is invalid, the method throws an exception and does not send a request to the 
        iNTERFACEWARE server. If the registration code is valid, this method registers 
        your machine.
        
      }
      procedure RegisterTrialHost(RegistrationCode : String; ExpiryDate : TDateTime);
      
      {: 
        Returns true if the machine has a valid Chameleon license.
        
        This method checks the license status of the target machine but does not send a 
        request to the iNTERFACEWARE server. It returns true if the machine has a valid 
        Chameleon license and false otherwise.
        
      }
      property IsLicensed : Boolean read GetIsLicensed;
      
      
      {: 
        When a Web proxy is required, this property specifies the IP address or hostname 
        for the Web proxy to use.
        
        A Web proxy is required when the local machine does not have direct access to 
        the Internet and needs to forward the registration requests through another 
        server. When it is required, use this property to specify the proxy's IP address 
        or hostname.
        
        A Web proxy server is a server that handles the HTTP requests of its clients by 
        forwarding its requests to other servers (for example, the iNTERFACEWARE 
        server). A client will connect to the Web proxy server to request a file. The 
        proxy server then connects to the specified server and requests the file on 
        behalf of the client.
        
        If your network requires your Internet connection to be set up via a Web proxy, 
        you must specify the proxy your network uses to forward HTTP requests to other 
        servers.
        
        By default, no proxy will be used. To unset a Web proxy, simply set this 
        property to an empty string.
        
      }
      property Proxy : String read GetProxy write SetProxy;
      
      
      {: 
        This method returns the unique machine ID of the machine to register.
        
      }
      property MachineId : String read GetMachineId;
      
   end;
   

{: 
  Fired when the connection is made with the server.
  
}
TCHMllpClientConnectedEvent =  procedure(Sender : TObject) of Object;

{: 
  Fired when the connection is closed.
  
}
TCHMllpClientClosedEvent =  procedure(Sender : TObject) of Object;

{: 
  Fired when a message is received.
  
  Does not include the HeaderBlock and TrailerBlock characters.
  
}
TCHMllpClientMessageReceivedEvent =  procedure(Sender : TObject; Message : String) of Object;

{: 
  Fired when data has been received that is not within the header and trailer 
  block.
  
  This is for dealing with an incorrect LLP implementation. The data is provided 
  to the user's application through this event. This allows it to be logged since 
  this is likely to be a bad error.
  
}
TCHMllpClientDataIgnoredEvent =  procedure(Sender : TObject; Data : String) of Object;

{: 
  Fired when we have an error with the TCP/IP connection.
  
  The context of when this is received is often determined by a previous 
  operation. The connection will always be closed once this event is received.
  
}
TCHMllpClientErrorReceivedEvent =  procedure(Sender : TObject; Description : String; Code : Integer) of Object;


{: 
  This class can be used to implement a Lower Layer Protocol (LLP) TCP/IP client 
  to connect to an LLP server.
  
  See http://www.interfaceware.com/manual/network_components.html for more 
  information.
  
}
type TCHMllpClient = class(TComponent)
   private
      {$IFNDEF HIDE}
      FHandle : TCHMclientHandle;
      FOnConnected : TCHMllpClientConnectedEvent;
      FOnClosed : TCHMllpClientClosedEvent;
      FOnMessageReceived : TCHMllpClientMessageReceivedEvent;
      FOnDataIgnored : TCHMllpClientDataIgnoredEvent;
      FOnErrorReceived : TCHMllpClientErrorReceivedEvent;
      
      {: 
        The characters that precede a message in a stream.
        
        The default value for this property is 0x0B as defined by the the LLP protocol.
        
        This property is used both for parsing out the incoming data stream and wrapping 
        outgoing messages.
        
        See also the SetHeaderBlock method.
        
      }
      function GetHeaderBlock : String;
      
      {: 
        Set method for headerBlock.
        
        See HeaderBlock.
        
      }
      procedure SetHeaderBlock(Value : String);
      
      {: 
        The characters that follow a message in a stream.
        
        The default value is the character 0x1C followed by 0x0D as defined by the LLP 
        protocol.
        
        This property is used both for parsing out the incoming data stream and wrapping 
        outgoing messages.
        
        See also the SetTrailerBlock method.
        
      }
      function GetTrailerBlock : String;
      
      {: 
        Set method for trailerBlock.
        
        See TrailerBlock.
        
      }
      procedure SetTrailerBlock(Value : String);
      
      {: 
        The status of the connection with the server.
        
        Returns true if the client is currently connected with the server.
        
      }
      function GetIsConnected : Boolean;
      {$ENDIF}
   public
      constructor Create(Owner : TComponent); override;
      destructor Destroy; override;
      
      {: 
         Send a quit message to the thread that the networking components are running 
        on. 
        
         Under Windows, this is a WM_QUIT message. All the networking components must be 
        created on a single thread with a running message loop. For more information, 
        see the RunMessageLoop method. 
        
      }
      procedure StopMessageLoop;
      
      {: 
        On Windows platforms all events are dispatched using the Windows message loop.
        
        Graphical Windows applications contain a running Windows message loop; 
        therefore, this function will not need to be called. Console applications 
        require this function to be called in order to start receiving events.
        
        Throws: Function will throw an exception only if the operating system reports an 
        error.
         Blocking: Function will block until StopMessageLoop is called, or (Windows 
        only) a WM_QUIT message is posted to the message loop on the thread that the 
        networking components are running on.
        
      }
      class procedure RunMessageLoop;
      
      {: 
        Make message printable by changing plain carriage return characters "\r" to 
        carriage return and newline characters "\r\n".
        
        HL7 messages use the carriage return character "\r" to separate the segments. 
        This makes it difficult to print them out in console windows because the cursor 
        overwrites each previous segment instead of moving on to a newline for each 
        segment. This method is intended to be a simple utility method to make it easier 
        to convert HL7 messages into a printable format. This function also allows the 
        output string to be truncated.
        
      }
      function MakeMessagePrintable(InputString : String; NewLine : String; MaximumLength : Integer) : String;
      
      {: 
        Connect to the given host and port number.
        
        Function requires a running message loop (see RunMessageLoop).
        
        Blocking: Function does not block. 
        Events: If the connection succeeds, a OnConnected event results. If the 
        connection fails, an OnErrorReceived event results, containing the error message 
        indicating why the connect attempt has failed.
        
      }
      procedure Connect(Host : String; Port : Integer);
      
      {: 
        Attempt to gracefully disconnect from the server.
        
        A graceful disconnect implies a shutdown to the write end of the connection, and 
        reading all data until the peer issues a close to its write end.
        
        Note that the peer may not actually close the write end of its connection 
        (although this is rare), which results in the connection never receiving the 
        close event to signify that the connection is closed. In this situation, it is 
        recommended that HardDisconnect be called to close the connection.
        
        Function requires a running message loop (see RunMessageLoop).
        
        Blocking: Function does not block. 
        Events: If the close succeeds (the peer closes the connection), a OnClosed event 
        results. If an error occurs during the close operation, an OnErrorReceived event 
        results, containing the error message indicating why the close attempt has 
        failed.
        
      }
      procedure Disconnect;
      
      {: 
        Ungracefully disconnect without doing TCP/IP handshaking with the server.
        
        If there is data pending on the network buffer to be sent, the peer will receive 
        a reset on the connection if it attempts to read or write to the connection.
        
        Blocking: Function blocks; upon returning, the connection is closed. 
        Events: None.
        
      }
      procedure HardDisconnect;
      
      {: 
        Send a message to the server, prefixed and postfixed by the current HeaderBlock 
        and TrailerBlock characters.
        
        The data is not guaranteed to be received after this function is called. All 
        data is buffered internally; completion of this call does not signify reception 
        of data.
        
        Function requires a running message loop (see RunMessageLoop).
        
        Blocking: Function does not block. 
        Events: If an error occurs during the send operation an OnErrorReceived event 
        results, containing the error message indicating why the send attempt has 
        failed.
        
      }
      procedure TransmitMessage(Message : String);
      
      {: 
        The characters that precede a message in a stream.
        
        The default value for this property is 0x0B as defined by the the LLP protocol.
        
        This property is used both for parsing out the incoming data stream and wrapping 
        outgoing messages.
        
      }
      property HeaderBlock : String read GetHeaderBlock write SetHeaderBlock;
      
      
      {: 
        The characters that follow a message in a stream.
        
        The default value is the character 0x1C followed by 0x0D as defined by the LLP 
        protocol.
        
        This property is used both for parsing out the incoming data stream and wrapping 
        outgoing messages.
        
      }
      property TrailerBlock : String read GetTrailerBlock write SetTrailerBlock;
      
      
      {: 
        The status of the connection with the server.
        
        Returns true if the client is currently connected with the server.
        
      }
      property IsConnected : Boolean read GetIsConnected;
      
   published
      {: 
        Fired when the connection is made with the server.
        
      }
      property OnConnected : TCHMllpClientConnectedEvent read FOnConnected write FOnConnected;
      
      {: 
        Fired when the connection is closed.
        
      }
      property OnClosed : TCHMllpClientClosedEvent read FOnClosed write FOnClosed;
      
      {: 
        Fired when a message is received.
        
        Does not include the HeaderBlock and TrailerBlock characters.
        
      }
      property OnMessageReceived : TCHMllpClientMessageReceivedEvent read FOnMessageReceived write FOnMessageReceived;
      
      {: 
        Fired when data has been received that is not within the header and trailer 
        block.
        
        This is for dealing with an incorrect LLP implementation. The data is provided 
        to the user's application through this event. This allows it to be logged since 
        this is likely to be a bad error.
        
      }
      property OnDataIgnored : TCHMllpClientDataIgnoredEvent read FOnDataIgnored write FOnDataIgnored;
      
      {: 
        Fired when we have an error with the TCP/IP connection.
        
        The context of when this is received is often determined by a previous 
        operation. The connection will always be closed once this event is received.
        
      }
      property OnErrorReceived : TCHMllpClientErrorReceivedEvent read FOnErrorReceived write FOnErrorReceived;
      
   end;
   


{: 
  This object is passed as an argument for many of the events of the TCHMllpServer 
  object.
  
  This class provides methods to interact with each discrete client connection.
  
  See http://www.interfaceware.com/manual/network_components.html for more 
  information.
  
}
type ICHMllpConnection = interface
   {$IFNDEF HIDE}
   
   {: 
      This property will resolve the remoteIpAddress to the host name if possible 
     using the operating system. Sometimes no host name resolution will be possible 
     at which point this function will return 'unknown'. 
     
     For the first time this method is called, it blocks until the name is resolved. 
     This means that if you make the call in the context of your message loop your 
     application will appear to freeze for a few seconds. 
     
     Subsequent calls will return immediately with a cached value. 
     
      For more advanced functionality you may wish to use other facilities to resolve 
     the host name outside of the functionality offered here. 
     
   }
   function GetRemoteHostName : String;
   
   {: 
     A unique ID for all connection objects.
     
     This ID is generated sequentially (i.e. 0, 1, 2, 3, etc.) for all connections 
     received from a LlpServer, beginning at 0 when the application starts. The ID is 
     globally unique and will allow users to identify connections from the same 
     client, or connections from different clients.
     
   }
   function GetId : Integer;
   
   {: 
     The IP address of the remote host in host byte order.
     
     Consider using RemoteIpAddressAsString instead for a more usable format.
     
   }
   function GetRemoteIpAddress : Integer;
   
   {: 
     The IP address of the remote host as a string.
     
     See also RemoteIpAddress.
     
   }
   function GetRemoteIpAddressAsString : String;
   {$ENDIF}
   
   {: 
     Send a message to the client, prefixed and postfixed by the current HeaderBlock 
     and TrailerBlock characters.
     
     The data is not guaranteed to be received after this function is called. All 
     data is buffered internally; completion of this call does not signify reception 
     of data.
     
     Function requires a running message loop (see RunMessageLoop).
     
     Blocking: Function does not block. 
     Events: If an error occurs during the send operation, an OnClientErrorReceived 
     event results, containing the error message indicating why the send attempt has 
     failed.
     
   }
   procedure TransmitMessage(Message : String);
   
   {: 
     Attempt to gracefully disconnect from the server.
     
     A graceful disconnect implies a shutdown to the write end of the connection, and 
     reading all data until the peer issues a close to its write end.
     
     Note that the peer may not actually close the write end of its connection 
     (although this is rare), which results in the connection never receiving the 
     close event to signify that the connection is closed. In this situation, it is 
     recommended that HardDisconnect be called to close the connection.
     
     Function requires a running message loop (see RunMessageLoop).
     
     Blocking: Function does not block. 
     Events: If the close succeeds (the peer closes the connection), a 
     OnClientDisconnected event results. If an error occurs during the close 
     operation, an OnClientErrorReceived event results, containing the error message 
     indicating why the close attempt failed.
     
   }
   procedure Disconnect;
   
   {: 
     Ungracefully disconnect without doing TCP/IP handshaking with the server.
     
     If there is data pending on the network buffer to be sent, the peer will receive 
     a reset on the connection if it attempts to read or write to the connection.
     
     Blocking: Function blocks; upon returning, the connection is closed. 
     Events: None.
     
   }
   procedure HardDisconnect;
   
   {: 
      This property will resolve the remoteIpAddress to the host name if possible 
     using the operating system. Sometimes no host name resolution will be possible 
     at which point this function will return 'unknown'. 
     
     For the first time this method is called, it blocks until the name is resolved. 
     This means that if you make the call in the context of your message loop your 
     application will appear to freeze for a few seconds. 
     
     Subsequent calls will return immediately with a cached value. 
     
      For more advanced functionality you may wish to use other facilities to resolve 
     the host name outside of the functionality offered here. 
     
   }
   property RemoteHostName : String read GetRemoteHostName;
   
   
   {: 
     A unique ID for all connection objects.
     
     This ID is generated sequentially (i.e. 0, 1, 2, 3, etc.) for all connections 
     received from a LlpServer, beginning at 0 when the application starts. The ID is 
     globally unique and will allow users to identify connections from the same 
     client, or connections from different clients.
     
   }
   property Id : Integer read GetId;
   
   
   {: 
     The IP address of the remote host in host byte order.
     
     Consider using RemoteIpAddressAsString instead for a more usable format.
     
   }
   property RemoteIpAddress : Integer read GetRemoteIpAddress;
   
   
   {: 
     The IP address of the remote host as a string.
     
     See also RemoteIpAddress.
     
   }
   property RemoteIpAddressAsString : String read GetRemoteIpAddressAsString;
   
   end;
   

{: 
  Fired when an issue occurs with the server socket itself rather than a specific 
  client.
  
}
TCHMllpServerErrorReceivedEvent =  procedure(Sender : TObject; ErrorMessage : String; Code : Integer) of Object;

{: 
  Fired when a connection is made to the server by a new client.
  
  This event is useful for logging.
  
}
TCHMllpServerClientConnectedEvent =  procedure(Sender : TObject; Client : ICHMllpConnection) of Object;

{: 
  Fired when a client disconnects from the server, or a previous Disconnect call 
  on a Connection succeeds.
  
  This event is useful for logging.
  
}
TCHMllpServerClientDisconnectedEvent =  procedure(Sender : TObject; Client : ICHMllpConnection) of Object;

{: 
  Fired when a networking error occurs with a specific client connection.
  
  The context on when this is received is often determined by a previous 
  operation, although that is not necessary. The connection will always be closed 
  once this event is received so there is no need to close the connection within 
  the application code.
  
}
TCHMllpServerClientErrorReceivedEvent =  procedure(Sender : TObject; Client : ICHMllpConnection; ErrorMessage : String; ErrorCode : Integer) of Object;

{: 
  Fired each time we receive a message from a connected HL7 client.
  
  Typically, the message should be processed and only then should an HL7 
  ACKnowledgement message be sent.
  
  Received without any Trailer or Header characters.
  
}
TCHMllpServerMessageReceivedEvent =  procedure(Sender : TObject; Client : ICHMllpConnection; Message : String) of Object;

{: 
  Fired when data has been received that is not within the header and trailer 
  block envelope.
  
  This is for dealing with an incorrect LLP implementation. The data is provided 
  to the user's application through this event. This allows it to be logged since 
  this is likely to be a very significant error with the counterparty's LLP 
  implementation.
  
}
TCHMllpServerDataIgnoredEvent =  procedure(Sender : TObject; Client : ICHMllpConnection; Data : String) of Object;


{: 
  This object implements a Lower Layer Protocol (LLP) TCP/IP server that listens 
  for connections from clients.
  
  Most of the network events that are raised provide a ICHMllpConnection 
  parameter, which gives methods to interact with the client connection that the 
  event is associated with.
  
  See http://www.interfaceware.com/manual/network_components.html for more 
  information.
  
}
type TCHMllpServer = class(TComponent)
   private
      {$IFNDEF HIDE}
      FHandle : TCHMlistenerHandle;
      FOnErrorReceived : TCHMllpServerErrorReceivedEvent;
      FOnClientConnected : TCHMllpServerClientConnectedEvent;
      FOnClientDisconnected : TCHMllpServerClientDisconnectedEvent;
      FOnClientErrorReceived : TCHMllpServerClientErrorReceivedEvent;
      FOnMessageReceived : TCHMllpServerMessageReceivedEvent;
      FOnDataIgnored : TCHMllpServerDataIgnoredEvent;
      
      {: 
        The port on which we are listening, set by the StartListen method.
        
      }
      function GetPort : Integer;
      
      {: 
        The local interface on which we are listening, used by the next call to the 
        StartListen method.
        
        The default is an empty string, which means the listener will listen on all 
        local interfaces.
        
        It must be in the Internet standard "." (dotted) notation (e.g. 127.0.0.1).
        
        See also the SetHost method.
        
      }
      function GetHost : String;
      
      {: 
        Set method for host.
        
        See Host.
        
      }
      procedure SetHost(Value : String);
      
      {: 
        The characters that precede a message in a stream.
        
        The default value for this property is 0x0B as defined by the the LLP protocol.
        
        This property is used both for parsing out the incoming data streams and 
        wrapping outgoing messages.
        
        See also the SetHeaderBlock method.
        
      }
      function GetHeaderBlock : String;
      
      {: 
        Set method for headerBlock.
        
        See HeaderBlock.
        
      }
      procedure SetHeaderBlock(Value : String);
      
      {: 
        The characters that follow a message in a stream.
        
        The default value is the character 0x1C followed by 0x0D as defined by the LLP 
        protocol.
        
        This property is used both for parsing out the incoming data streams and 
        wrapping outgoing messages.
        
        See also the SetTrailerBlock method.
        
      }
      function GetTrailerBlock : String;
      
      {: 
        Set method for trailerBlock.
        
        See TrailerBlock.
        
      }
      procedure SetTrailerBlock(Value : String);
      
      {: 
        The status of the listening state.
        
        Returns true if the server is currently listening for connections. 
        
        Blocking: Function blocks until the listening state is determined. 
        Events: None.
        
      }
      function GetIsListening : Boolean;
      
      {: 
        Representation of the collection of remote Clients that are connected to this 
        server.
        
        This list changes in size as connections are received or closed, or if an error 
        occurs on the connection (which implies a close).
        
        Users cannot change the size or append to the contents of this collection since 
        it is managed internally.
        
        References to ICHMllpConnection objects within this collection should not be 
        maintained. Instead, always access ICHMllpConnection objects through this 
        collection interface, or the event to which ICHMllpConnection objects are passed 
        as parameters.
        
      }
      function GetClient(ClientIndex : Integer) : ICHMllpConnection;
      {$ENDIF}
   public
      constructor Create(Owner : TComponent); override;
      destructor Destroy; override;
      
      {: 
        Returns the total number of Clients.
        
        See also the Client property.
        
      }
      function CountOfClient : Integer;
      
      {: 
         This will send a quit message to the thread that the networking components are 
        running on. 
        
         Under Windows, this is a WM_QUIT message. All the networking components must be 
        created on a single thread with a running message loop. For more information, 
        see the RunMessageLoop method. 
        
      }
      procedure StopMessageLoop;
      
      {: 
        On Windows platforms all events are dispatched using the Windows message loop.
        
        Graphical Windows applications contain a running Windows message loop; 
        therefore, this function will not need to be called. Console applications 
        require this function to be called in order to start receiving events.
        
        Throws: Function will throw an exception only if the operating system reports an 
        error.
         Blocking: Function will block until StopMessageLoop is called, or (Windows 
        only) a WM_QUIT message is posted to the message loop on the thread that the 
        networking components are running on.
        
      }
      class procedure RunMessageLoop;
      
      {: 
        Attempt to listen on the supplied port for incoming client connections.
        
        This function requires a running message loop (see RunMessageLoop) in order to 
        receive connections.
        
        Blocking: Function blocks until a port can be successfully listened on or an 
        error occurs. 
        Throws: Function will throw an exception if the port is in use, or the operating 
        system reports any other error. 
        Events: None.
        
        By default, the listener will listen on all local interfaces. To make it listen 
        on a specific local interface use the Host property to set the IP address of the 
        interface to use.
        
      }
      procedure StartListen(Port : Integer);
      
      {: 
        Make message printable by changing plain carriage return characters "\r" to 
        carriage return and newline characters "\r\n".
        
        HL7 messages use the carriage return character "\r" to separate the segments. 
        This makes it difficult to print them out in console windows because the cursor 
        overwrites each previous segment instead of moving on to a newline for each 
        segment. This method is intended to be a simple utility method to make it easier 
        to convert HL7 messages into a printable format. This function also allows the 
        output string to be truncated.
        
      }
      function MakeMessagePrintable(InputString : String; NewLine : String; MaximumLength : Integer) : String;
      
      {: 
        Stop the component from listening for new connections.
        
        Note that any existing connections that have already been established are NOT 
        closed. In order to properly disconnect existing connections as well as stop 
        listening, use DisconnectAllClients.
        
        Blocking: Function blocks until the stop listen succeeds. 
        Throws: Function does not throw any exceptions. 
        Events: None.
        
      }
      procedure StopListen;
      
      {: 
        Stop listening on the port for new connections, and initiate a Disconnect on all 
        open connections.
        
        This function requires a running message loop (see RunMessageLoop).
        
        Blocking: Function blocks until Disconnect is called on all open connections. 
        This implies that the connections are not necessarily closed when this function 
        returns. 
        Events: None.
        
      }
      procedure DisconnectAllClients;
      
      {: 
        The port on which we are listening, set by the StartListen method.
        
      }
      property Port : Integer read GetPort;
      
      
      {: 
        The local interface on which we are listening, used by the next call to the 
        StartListen method.
        
        The default is an empty string, which means the listener will listen on all 
        local interfaces.
        
        It must be in the Internet standard "." (dotted) notation (e.g. 127.0.0.1).
        
      }
      property Host : String read GetHost write SetHost;
      
      
      {: 
        The characters that precede a message in a stream.
        
        The default value for this property is 0x0B as defined by the the LLP protocol.
        
        This property is used both for parsing out the incoming data streams and 
        wrapping outgoing messages.
        
      }
      property HeaderBlock : String read GetHeaderBlock write SetHeaderBlock;
      
      
      {: 
        The characters that follow a message in a stream.
        
        The default value is the character 0x1C followed by 0x0D as defined by the LLP 
        protocol.
        
        This property is used both for parsing out the incoming data streams and 
        wrapping outgoing messages.
        
      }
      property TrailerBlock : String read GetTrailerBlock write SetTrailerBlock;
      
      
      {: 
        The status of the listening state.
        
        Returns true if the server is currently listening for connections. 
        
        Blocking: Function blocks until the listening state is determined. 
        Events: None.
        
      }
      property IsListening : Boolean read GetIsListening;
      
      
      {: 
        Representation of the collection of remote Clients that are connected to this 
        server.
        
        This list changes in size as connections are received or closed, or if an error 
        occurs on the connection (which implies a close).
        
        Users cannot change the size or append to the contents of this collection since 
        it is managed internally.
        
        References to ICHMllpConnection objects within this collection should not be 
        maintained. Instead, always access ICHMllpConnection objects through this 
        collection interface, or the event to which ICHMllpConnection objects are passed 
        as parameters.
        
      }
      property Client[Index : Integer] : ICHMllpConnection read GetClient;
      
   published
      {: 
        Fired when an issue occurs with the server socket itself rather than a specific 
        client.
        
      }
      property OnErrorReceived : TCHMllpServerErrorReceivedEvent read FOnErrorReceived write FOnErrorReceived;
      
      {: 
        Fired when a connection is made to the server by a new client.
        
        This event is useful for logging.
        
      }
      property OnClientConnected : TCHMllpServerClientConnectedEvent read FOnClientConnected write FOnClientConnected;
      
      {: 
        Fired when a client disconnects from the server, or a previous Disconnect call 
        on a Connection succeeds.
        
        This event is useful for logging.
        
      }
      property OnClientDisconnected : TCHMllpServerClientDisconnectedEvent read FOnClientDisconnected write FOnClientDisconnected;
      
      {: 
        Fired when a networking error occurs with a specific client connection.
        
        The context on when this is received is often determined by a previous 
        operation, although that is not necessary. The connection will always be closed 
        once this event is received so there is no need to close the connection within 
        the application code.
        
      }
      property OnClientErrorReceived : TCHMllpServerClientErrorReceivedEvent read FOnClientErrorReceived write FOnClientErrorReceived;
      
      {: 
        Fired each time we receive a message from a connected HL7 client.
        
        Typically, the message should be processed and only then should an HL7 
        ACKnowledgement message be sent.
        
        Received without any Trailer or Header characters.
        
      }
      property OnMessageReceived : TCHMllpServerMessageReceivedEvent read FOnMessageReceived write FOnMessageReceived;
      
      {: 
        Fired when data has been received that is not within the header and trailer 
        block envelope.
        
        This is for dealing with an incorrect LLP implementation. The data is provided 
        to the user's application through this event. This allows it to be logged since 
        this is likely to be a very significant error with the counterparty's LLP 
        implementation.
        
      }
      property OnDataIgnored : TCHMllpServerDataIgnoredEvent read FOnDataIgnored write FOnDataIgnored;
      
   end;
   


{: 
  This class contains functions to change the current encoding used internally by 
  Chameleon at runtime. Changes made here affect all currently instantiated 
  Engines and Chameleon network connections.
  
   The locale set here is used to determine how to 
  
   1) Translate .NET and Java unicode strings to and from the internal 
  representation used in Chameleon for both Engine and network operations. See 
  http://www.interfaceware.com/manual/setting_encoding.html for more details.
  
   2) Determine how strings are parsed with an Engine.
  
}
type TCHMchameleonEncoding = class(TComponent)
   private
      {$IFNDEF HIDE}
      FHandle : TCHMchameleonEncodingHandle;
      {$ENDIF}
   public
      constructor Create(Owner : TComponent); override;
      destructor Destroy; override;
      
      {: 
        Get the translation locale.
        
        The initial value is extracted from the default system locale.
        
      }
      class function GetCurrentTranslationLocale : String;
      
      {: 
        Set the translation locale. See 
        http://www.interfaceware.com/manual/setting_encoding.html for more details 
        
      }
      class procedure SetCurrentTranslationLocale(NewLocale : String);
      
      {: 
        Get the total number of supported translation locales. 
        
      }
      class function CountOfTranslationLocale : Integer;
      
      {: 
        Get a translation locale at a particular index.
        
      }
      class function TranslationLocale(LocaleIndex : Integer) : String;
   end;
   

function CHMllpConnectionCreateObject : ICHMllpConnection;
function CHMllpConnectionCreateObjectWithHandle(Handle : TCHMconnectionHandle) : ICHMllpConnection;

{$IFNDEF HIDE}
procedure Register;
{$ENDIF}

implementation

uses CHMinterface;

procedure Register;
begin
   RegisterComponents('Chameleon', [TCHMlicense, TCHMllpClient, TCHMllpServer, TCHMchameleonEncoding]);
end;

function CHMlicenseCreate
(
   ReturnValue : TCHMlicenseHandlePtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseCreate@4';

function CHMlicenseAddRef
(
   Handle : TCHMlicenseHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseAddRef@4';

function CHMlicenseRelease
(
   Handle : TCHMlicenseHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseRelease@4';

function CHMlicenseGetIsLicensed
(
   Handle : TCHMlicenseHandle;
   ReturnValue : TBooleanPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseGetIsLicensed@8';

function CHMlicenseGetProxy
(
   Handle : TCHMlicenseHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseGetProxy@8';

function CHMlicenseSetProxy
(
   Handle : TCHMlicenseHandle;
   Value : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseSetProxy@8';

function CHMlicenseGetMachineId
(
   Handle : TCHMlicenseHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseGetMachineId@8';

function CHMlicenseFetchRegistrationCode
(
   Handle : TCHMlicenseHandle;
   MachineId : PChar;
   EmailAddress : PChar;
   Password : PChar;
   Description : PChar;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseFetchRegistrationCode@24';

function CHMlicenseRegisterHost
(
   Handle : TCHMlicenseHandle;
   RegistrationCode : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseRegisterHost@8';

function CHMlicenseFetchTrialRegistrationCode
(
   Handle : TCHMlicenseHandle;
   ExpiryDate : TDoublePtr;
   MachineId : PChar;
   EmailAddress : PChar;
   Password : PChar;
   Description : PChar;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseFetchTrialRegistrationCode@28';

function CHMlicenseRegisterTrialHost
(
   Handle : TCHMlicenseHandle;
   RegistrationCode : PChar;
   ExpiryDate : Double
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlicenseRegisterTrialHost@16';

constructor TCHMlicense.Create(Owner: TComponent);
begin
   FHandle := nil;
   inherited Create(Owner);
   DLLcheckResult(CHMlicenseCreate(Addr(FHandle)));
   DLLcheckResult(CHMlicenseAddRef(FHandle));
end;

destructor TCHMlicense.Destroy;
begin
   DLLcheckResult(CHMlicenseRelease(FHandle));
   inherited Destroy;
end;

function TCHMlicense.GetIsLicensed : Boolean;
var
   ResultValue : Boolean;
begin
   DLLcheckResult(CHMlicenseGetIsLicensed(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMlicense.GetProxy : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlicenseGetProxy(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMlicense.SetProxy
(
   Value : String
);
begin
   DLLcheckResult(CHMlicenseSetProxy(FHandle, PChar(Value)));
end;

function TCHMlicense.GetMachineId : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlicenseGetMachineId(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMlicense.FetchRegistrationCode
(
   MachineId : String; EmailAddress : String; Password : String; Description : String
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlicenseFetchRegistrationCode(FHandle, PChar(MachineId), PChar(EmailAddress), PChar(Password), PChar(Description), Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMlicense.RegisterHost
(
   RegistrationCode : String
);
begin
   DLLcheckResult(CHMlicenseRegisterHost(FHandle, PChar(RegistrationCode)));
end;

function TCHMlicense.FetchTrialRegistrationCode
(
   var ExpiryDate : TDateTime; MachineId : String; EmailAddress : String; Password : String; Description : String
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlicenseFetchTrialRegistrationCode(FHandle, Addr(ExpiryDate), PChar(MachineId), PChar(EmailAddress), PChar(Password), PChar(Description), Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMlicense.RegisterTrialHost
(
   RegistrationCode : String; ExpiryDate : TDateTime
);
begin
   DLLcheckResult(CHMlicenseRegisterTrialHost(FHandle, PChar(RegistrationCode), ExpiryDate));
end;

function CHMclientCreate
(
   ReturnValue : TCHMclientHandlePtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientCreate@4';

function CHMclientAddRef
(
   Handle : TCHMclientHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientAddRef@4';

function CHMclientRelease
(
   Handle : TCHMclientHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientRelease@4';

function CHMclientAddListener
(
   Handle : TCHMclientHandle;
   pListener : Pointer
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientAddListener@8';

function CHMclientRemoveListener
(
   Handle : TCHMclientHandle;
   pListener : Pointer
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientRemoveListener@8';

function CHMclientGetHeaderBlock
(
   Handle : TCHMclientHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientGetHeaderBlock@8';

function CHMclientSetHeaderBlock
(
   Handle : TCHMclientHandle;
   Value : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientSetHeaderBlock@8';

function CHMclientGetTrailerBlock
(
   Handle : TCHMclientHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientGetTrailerBlock@8';

function CHMclientSetTrailerBlock
(
   Handle : TCHMclientHandle;
   Value : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientSetTrailerBlock@8';

function CHMclientGetIsConnected
(
   Handle : TCHMclientHandle;
   ReturnValue : TBooleanPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientGetIsConnected@8';

function CHMclientStopMessageLoop
(
   Handle : TCHMclientHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientStopMessageLoop@4';

function CHMclientRunMessageLoop
 : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientRunMessageLoop@0';

function CHMclientMakePrintable
(
   Handle : TCHMclientHandle;
   InputString : PChar;
   NewLine : PChar;
   MaximumLength : Integer;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientMakePrintable@20';

function CHMclientConnect
(
   Handle : TCHMclientHandle;
   Host : PChar;
   Port : Integer
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientConnect@12';

function CHMclientClose
(
   Handle : TCHMclientHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientClose@4';

function CHMclientHardClose
(
   Handle : TCHMclientHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientHardClose@4';

function CHMclientSendMessage
(
   Handle : TCHMclientHandle;
   Message : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMclientSendMessage@8';

type TCHMclientOnConnectCallback = function
(
   Handle : TCHMllpClient
) : TCHMresultHandle; stdcall;

function CHMclientSetConnectCallback(Callback : TCHMclientOnConnectCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMclientSetConnectCallback@4';

type TCHMclientOnClosedCallback = function
(
   Handle : TCHMllpClient
) : TCHMresultHandle; stdcall;

function CHMclientSetClosedCallback(Callback : TCHMclientOnClosedCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMclientSetClosedCallback@4';

type TCHMclientOnMessageCallback = function
(
   Handle : TCHMllpClient;
   Message : PChar
) : TCHMresultHandle; stdcall;

function CHMclientSetMessageCallback(Callback : TCHMclientOnMessageCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMclientSetMessageCallback@4';

type TCHMclientOnIgnoredDataCallback = function
(
   Handle : TCHMllpClient;
   Data : PChar
) : TCHMresultHandle; stdcall;

function CHMclientSetIgnoredDataCallback(Callback : TCHMclientOnIgnoredDataCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMclientSetIgnoredDataCallback@4';

type TCHMclientOnErrorCallback = function
(
   Handle : TCHMllpClient;
   Description : PChar;
   Code : Integer
) : TCHMresultHandle; stdcall;

function CHMclientSetErrorCallback(Callback : TCHMclientOnErrorCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMclientSetErrorCallback@4';

constructor TCHMllpClient.Create(Owner: TComponent);
begin
   FHandle := nil;
   inherited Create(Owner);
   DLLcheckResult(CHMclientCreate(Addr(FHandle)));
   DLLcheckResult(CHMclientAddRef(FHandle));
   DLLcheckResult(CHMclientAddListener(FHandle, Self));
end;

destructor TCHMllpClient.Destroy;
begin
   DLLcheckResult(CHMclientRemoveListener(FHandle, Self));
   DLLcheckResult(CHMclientRelease(FHandle));
   inherited Destroy;
end;

function TCHMllpClient.GetHeaderBlock : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMclientGetHeaderBlock(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpClient.SetHeaderBlock
(
   Value : String
);
begin
   DLLcheckResult(CHMclientSetHeaderBlock(FHandle, PChar(Value)));
end;

function TCHMllpClient.GetTrailerBlock : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMclientGetTrailerBlock(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpClient.SetTrailerBlock
(
   Value : String
);
begin
   DLLcheckResult(CHMclientSetTrailerBlock(FHandle, PChar(Value)));
end;

function TCHMllpClient.GetIsConnected : Boolean;
var
   ResultValue : Boolean;
begin
   DLLcheckResult(CHMclientGetIsConnected(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpClient.StopMessageLoop;
begin
   DLLcheckResult(CHMclientStopMessageLoop(FHandle));
end;

class procedure TCHMllpClient.RunMessageLoop;
begin
   DLLcheckResult(CHMclientRunMessageLoop());
end;

function TCHMllpClient.MakeMessagePrintable
(
   InputString : String; NewLine : String; MaximumLength : Integer
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMclientMakePrintable(FHandle, PChar(InputString), PChar(NewLine), MaximumLength, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpClient.Connect
(
   Host : String; Port : Integer
);
begin
   DLLcheckResult(CHMclientConnect(FHandle, PChar(Host), Port));
end;

procedure TCHMllpClient.Disconnect;
begin
   DLLcheckResult(CHMclientClose(FHandle));
end;

procedure TCHMllpClient.HardDisconnect;
begin
   DLLcheckResult(CHMclientHardClose(FHandle));
end;

procedure TCHMllpClient.TransmitMessage
(
   Message : String
);
begin
   DLLcheckResult(CHMclientSendMessage(FHandle, PChar(Message)));
end;

function CHMllpClientConnectedCallback
(
   Receiver : TCHMllpClient
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnConnected) then
   begin
      Receiver.FOnConnected(Receiver);
   end;
   Result := nil;
end;

function CHMllpClientClosedCallback
(
   Receiver : TCHMllpClient
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnClosed) then
   begin
      Receiver.FOnClosed(Receiver);
   end;
   Result := nil;
end;

function CHMllpClientMessageReceivedCallback
(
   Receiver : TCHMllpClient;
   Message : PChar
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnMessageReceived) then
   begin
      Receiver.FOnMessageReceived(Receiver, PChar(Message));
   end;
   Result := nil;
end;

function CHMllpClientDataIgnoredCallback
(
   Receiver : TCHMllpClient;
   Data : PChar
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnDataIgnored) then
   begin
      Receiver.FOnDataIgnored(Receiver, PChar(Data));
   end;
   Result := nil;
end;

function CHMllpClientErrorReceivedCallback
(
   Receiver : TCHMllpClient;
   Description : PChar;
   Code : Integer
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnErrorReceived) then
   begin
      Receiver.FOnErrorReceived(Receiver, PChar(Description), Code);
   end;
   Result := nil;
end;

{: 
  This object is passed as an argument for many of the events of the TCHMllpServer 
  object.
  
  This class provides methods to interact with each discrete client connection.
  
  See http://www.interfaceware.com/manual/network_components.html for more 
  information.
  
}
type TCHMllpConnection = class(TCHMiunknown, ICHMllpConnection)
   private
      FHandle : TCHMconnectionHandle;
   public
      constructor Create;
      constructor CreateWithHandle(Handle : TCHMconnectionHandle);
      function _AddRef : Integer; override; stdcall;
      function _Release : Integer; override; stdcall;
      destructor Destroy; override;
      {$IFNDEF HIDE}
      
      {: 
         This property will resolve the remoteIpAddress to the host name if possible 
        using the operating system. Sometimes no host name resolution will be possible 
        at which point this function will return 'unknown'. 
        
        For the first time this method is called, it blocks until the name is resolved. 
        This means that if you make the call in the context of your message loop your 
        application will appear to freeze for a few seconds. 
        
        Subsequent calls will return immediately with a cached value. 
        
         For more advanced functionality you may wish to use other facilities to resolve 
        the host name outside of the functionality offered here. 
        
      }
      function GetRemoteHostName : String;
      
      {: 
        A unique ID for all connection objects.
        
        This ID is generated sequentially (i.e. 0, 1, 2, 3, etc.) for all connections 
        received from a LlpServer, beginning at 0 when the application starts. The ID is 
        globally unique and will allow users to identify connections from the same 
        client, or connections from different clients.
        
      }
      function GetId : Integer;
      
      {: 
        The IP address of the remote host in host byte order.
        
        Consider using RemoteIpAddressAsString instead for a more usable format.
        
      }
      function GetRemoteIpAddress : Integer;
      
      {: 
        The IP address of the remote host as a string.
        
        See also RemoteIpAddress.
        
      }
      function GetRemoteIpAddressAsString : String;
      {$ENDIF}
      
      {: 
        Send a message to the client, prefixed and postfixed by the current HeaderBlock 
        and TrailerBlock characters.
        
        The data is not guaranteed to be received after this function is called. All 
        data is buffered internally; completion of this call does not signify reception 
        of data.
        
        Function requires a running message loop (see RunMessageLoop).
        
        Blocking: Function does not block. 
        Events: If an error occurs during the send operation, an OnClientErrorReceived 
        event results, containing the error message indicating why the send attempt has 
        failed.
        
      }
      procedure TransmitMessage(Message : String);
      
      {: 
        Attempt to gracefully disconnect from the server.
        
        A graceful disconnect implies a shutdown to the write end of the connection, and 
        reading all data until the peer issues a close to its write end.
        
        Note that the peer may not actually close the write end of its connection 
        (although this is rare), which results in the connection never receiving the 
        close event to signify that the connection is closed. In this situation, it is 
        recommended that HardDisconnect be called to close the connection.
        
        Function requires a running message loop (see RunMessageLoop).
        
        Blocking: Function does not block. 
        Events: If the close succeeds (the peer closes the connection), a 
        OnClientDisconnected event results. If an error occurs during the close 
        operation, an OnClientErrorReceived event results, containing the error message 
        indicating why the close attempt failed.
        
      }
      procedure Disconnect;
      
      {: 
        Ungracefully disconnect without doing TCP/IP handshaking with the server.
        
        If there is data pending on the network buffer to be sent, the peer will receive 
        a reset on the connection if it attempts to read or write to the connection.
        
        Blocking: Function blocks; upon returning, the connection is closed. 
        Events: None.
        
      }
      procedure HardDisconnect;
      
      {: 
         This property will resolve the remoteIpAddress to the host name if possible 
        using the operating system. Sometimes no host name resolution will be possible 
        at which point this function will return 'unknown'. 
        
        For the first time this method is called, it blocks until the name is resolved. 
        This means that if you make the call in the context of your message loop your 
        application will appear to freeze for a few seconds. 
        
        Subsequent calls will return immediately with a cached value. 
        
         For more advanced functionality you may wish to use other facilities to resolve 
        the host name outside of the functionality offered here. 
        
      }
      property RemoteHostName : String read GetRemoteHostName;
      
      
      {: 
        A unique ID for all connection objects.
        
        This ID is generated sequentially (i.e. 0, 1, 2, 3, etc.) for all connections 
        received from a LlpServer, beginning at 0 when the application starts. The ID is 
        globally unique and will allow users to identify connections from the same 
        client, or connections from different clients.
        
      }
      property Id : Integer read GetId;
      
      
      {: 
        The IP address of the remote host in host byte order.
        
        Consider using RemoteIpAddressAsString instead for a more usable format.
        
      }
      property RemoteIpAddress : Integer read GetRemoteIpAddress;
      
      
      {: 
        The IP address of the remote host as a string.
        
        See also RemoteIpAddress.
        
      }
      property RemoteIpAddressAsString : String read GetRemoteIpAddressAsString;
      
   end;
   
function CHMconnectionCreate
(
   ReturnValue : TCHMconnectionHandlePtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionCreate@4';

function CHMconnectionAddRef
(
   Handle : TCHMconnectionHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionAddRef@4';

function CHMconnectionRelease
(
   Handle : TCHMconnectionHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionRelease@4';

function CHMconnectionGetRemoteHostName
(
   Handle : TCHMconnectionHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionGetRemoteHostName@8';

function CHMconnectionGetId
(
   Handle : TCHMconnectionHandle;
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionGetId@8';

function CHMconnectionGetRemoteIpAddress
(
   Handle : TCHMconnectionHandle;
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionGetRemoteIpAddress@8';

function CHMconnectionGetRemoteIpAddressAsString
(
   Handle : TCHMconnectionHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionGetRemoteIpAddressAsString@8';

function CHMconnectionSendMessage
(
   Handle : TCHMconnectionHandle;
   Message : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionSendMessage@8';

function CHMconnectionClose
(
   Handle : TCHMconnectionHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionClose@4';

function CHMconnectionHardClose
(
   Handle : TCHMconnectionHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMconnectionHardClose@4';

constructor TCHMllpConnection.Create();
begin
   inherited Create;
   FHandle := nil;
   DLLcheckResult(CHMconnectionCreate(Addr(FHandle)));
end;

constructor TCHMllpConnection.CreateWithHandle(Handle : TCHMconnectionHandle);
begin
   inherited Create;
   FHandle := Handle;
end;

function TCHMllpConnection._AddRef : Integer;
begin
   Result := inherited _AddRef;
   DLLcheckResult(CHMconnectionAddRef(FHandle));
end;

function TCHMllpConnection._Release : Integer;
begin
   DLLcheckResult(CHMconnectionRelease(FHandle));
Result := inherited _Release;
end;

destructor TCHMllpConnection.Destroy;
begin
   inherited Destroy;
end;

function CHMllpConnectionCreateObject : ICHMllpConnection;
begin
   Result := TCHMllpConnection.Create();
end;

function CHMllpConnectionCreateObjectWithHandle(Handle : TCHMconnectionHandle) : ICHMllpConnection;
begin
   Result := TCHMllpConnection.CreateWithHandle(Handle);
end;

function TCHMllpConnection.GetRemoteHostName : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMconnectionGetRemoteHostName(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMllpConnection.GetId : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMconnectionGetId(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMllpConnection.GetRemoteIpAddress : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMconnectionGetRemoteIpAddress(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMllpConnection.GetRemoteIpAddressAsString : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMconnectionGetRemoteIpAddressAsString(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpConnection.TransmitMessage
(
   Message : String
);
begin
   DLLcheckResult(CHMconnectionSendMessage(FHandle, PChar(Message)));
end;

procedure TCHMllpConnection.Disconnect;
begin
   DLLcheckResult(CHMconnectionClose(FHandle));
end;

procedure TCHMllpConnection.HardDisconnect;
begin
   DLLcheckResult(CHMconnectionHardClose(FHandle));
end;

function CHMlistenerCreate
(
   ReturnValue : TCHMlistenerHandlePtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerCreate@4';

function CHMlistenerAddRef
(
   Handle : TCHMlistenerHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerAddRef@4';

function CHMlistenerRelease
(
   Handle : TCHMlistenerHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerRelease@4';

function CHMlistenerAddListener
(
   Handle : TCHMlistenerHandle;
   pListener : Pointer
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerAddListener@8';

function CHMlistenerRemoveListener
(
   Handle : TCHMlistenerHandle;
   pListener : Pointer
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerRemoveListener@8';

function CHMlistenerGetPort
(
   Handle : TCHMlistenerHandle;
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerGetPort@8';

function CHMlistenerGetHost
(
   Handle : TCHMlistenerHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerGetHost@8';

function CHMlistenerSetHost
(
   Handle : TCHMlistenerHandle;
   Value : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerSetHost@8';

function CHMlistenerGetHeaderBlock
(
   Handle : TCHMlistenerHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerGetHeaderBlock@8';

function CHMlistenerSetHeaderBlock
(
   Handle : TCHMlistenerHandle;
   Value : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerSetHeaderBlock@8';

function CHMlistenerGetTrailerBlock
(
   Handle : TCHMlistenerHandle;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerGetTrailerBlock@8';

function CHMlistenerSetTrailerBlock
(
   Handle : TCHMlistenerHandle;
   Value : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerSetTrailerBlock@8';

function CHMlistenerGetIsListening
(
   Handle : TCHMlistenerHandle;
   ReturnValue : TBooleanPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerGetIsListening@8';

function CHMlistenerGetConnection
(
   Handle : TCHMlistenerHandle;
   ClientIndex : Integer;
   ReturnValue : TCHMconnectionHandlePtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerGetConnection@12';

function CHMlistenerCountOfConnection
(
   Handle : TCHMlistenerHandle;
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerCountOfConnection@8';

function CHMlistenerStopMessageLoop
(
   Handle : TCHMlistenerHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerStopMessageLoop@4';

function CHMlistenerRunMessageLoop
 : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerRunMessageLoop@0';

function CHMlistenerStartListen
(
   Handle : TCHMlistenerHandle;
   Port : Integer
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerStartListen@8';

function CHMlistenerMakePrintable
(
   Handle : TCHMlistenerHandle;
   InputString : PChar;
   NewLine : PChar;
   MaximumLength : Integer;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerMakePrintable@20';

function CHMlistenerStopListen
(
   Handle : TCHMlistenerHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerStopListen@4';

function CHMlistenerCloseAll
(
   Handle : TCHMlistenerHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMlistenerCloseAll@4';

type TCHMlistenerOnListenerErrorCallback = function
(
   Handle : TCHMllpServer;
   ErrorMessage : PChar;
   Code : Integer
) : TCHMresultHandle; stdcall;

function CHMlistenerSetListenerErrorCallback(Callback : TCHMlistenerOnListenerErrorCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMlistenerSetListenerErrorCallback@4';

type TCHMlistenerOnConnectCallback = function
(
   Handle : TCHMllpServer;
   Client : TCHMconnectionHandle
) : TCHMresultHandle; stdcall;

function CHMlistenerSetConnectCallback(Callback : TCHMlistenerOnConnectCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMlistenerSetConnectCallback@4';

type TCHMlistenerOnDisconnectCallback = function
(
   Handle : TCHMllpServer;
   Client : TCHMconnectionHandle
) : TCHMresultHandle; stdcall;

function CHMlistenerSetDisconnectCallback(Callback : TCHMlistenerOnDisconnectCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMlistenerSetDisconnectCallback@4';

type TCHMlistenerOnConnectionErrorCallback = function
(
   Handle : TCHMllpServer;
   Client : TCHMconnectionHandle;
   ErrorMessage : PChar;
   ErrorCode : Integer
) : TCHMresultHandle; stdcall;

function CHMlistenerSetConnectionErrorCallback(Callback : TCHMlistenerOnConnectionErrorCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMlistenerSetConnectionErrorCallback@4';

type TCHMlistenerOnMessageCallback = function
(
   Handle : TCHMllpServer;
   Client : TCHMconnectionHandle;
   Message : PChar
) : TCHMresultHandle; stdcall;

function CHMlistenerSetMessageCallback(Callback : TCHMlistenerOnMessageCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMlistenerSetMessageCallback@4';

type TCHMlistenerOnIgnoredDataCallback = function
(
   Handle : TCHMllpServer;
   Client : TCHMconnectionHandle;
   Data : PChar
) : TCHMresultHandle; stdcall;

function CHMlistenerSetIgnoredDataCallback(Callback : TCHMlistenerOnIgnoredDataCallback)
: TCHMresultHandle; stdcall; external 'NETDLL2.dll' name '_CHMlistenerSetIgnoredDataCallback@4';

constructor TCHMllpServer.Create(Owner: TComponent);
begin
   FHandle := nil;
   inherited Create(Owner);
   DLLcheckResult(CHMlistenerCreate(Addr(FHandle)));
   DLLcheckResult(CHMlistenerAddRef(FHandle));
   DLLcheckResult(CHMlistenerAddListener(FHandle, Self));
end;

destructor TCHMllpServer.Destroy;
begin
   DLLcheckResult(CHMlistenerRemoveListener(FHandle, Self));
   DLLcheckResult(CHMlistenerRelease(FHandle));
   inherited Destroy;
end;

function TCHMllpServer.GetPort : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMlistenerGetPort(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMllpServer.GetHost : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlistenerGetHost(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpServer.SetHost
(
   Value : String
);
begin
   DLLcheckResult(CHMlistenerSetHost(FHandle, PChar(Value)));
end;

function TCHMllpServer.GetHeaderBlock : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlistenerGetHeaderBlock(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpServer.SetHeaderBlock
(
   Value : String
);
begin
   DLLcheckResult(CHMlistenerSetHeaderBlock(FHandle, PChar(Value)));
end;

function TCHMllpServer.GetTrailerBlock : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlistenerGetTrailerBlock(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpServer.SetTrailerBlock
(
   Value : String
);
begin
   DLLcheckResult(CHMlistenerSetTrailerBlock(FHandle, PChar(Value)));
end;

function TCHMllpServer.GetIsListening : Boolean;
var
   ResultValue : Boolean;
begin
   DLLcheckResult(CHMlistenerGetIsListening(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TCHMllpServer.GetClient
(
   ClientIndex : Integer
) : ICHMllpConnection;
var
   ResultValue : TCHMconnectionHandle;
begin
   DLLcheckResult(CHMlistenerGetConnection(FHandle, ClientIndex, Addr(ResultValue)));
   Result := CHMllpConnectionCreateObjectWithHandle(ResultValue);
end;

function TCHMllpServer.CountOfClient : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMlistenerCountOfConnection(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpServer.StopMessageLoop;
begin
   DLLcheckResult(CHMlistenerStopMessageLoop(FHandle));
end;

class procedure TCHMllpServer.RunMessageLoop;
begin
   DLLcheckResult(CHMlistenerRunMessageLoop());
end;

procedure TCHMllpServer.StartListen
(
   Port : Integer
);
begin
   DLLcheckResult(CHMlistenerStartListen(FHandle, Port));
end;

function TCHMllpServer.MakeMessagePrintable
(
   InputString : String; NewLine : String; MaximumLength : Integer
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMlistenerMakePrintable(FHandle, PChar(InputString), PChar(NewLine), MaximumLength, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure TCHMllpServer.StopListen;
begin
   DLLcheckResult(CHMlistenerStopListen(FHandle));
end;

procedure TCHMllpServer.DisconnectAllClients;
begin
   DLLcheckResult(CHMlistenerCloseAll(FHandle));
end;

function CHMllpServerErrorReceivedCallback
(
   Receiver : TCHMllpServer;
   ErrorMessage : PChar;
   Code : Integer
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnErrorReceived) then
   begin
      Receiver.FOnErrorReceived(Receiver, PChar(ErrorMessage), Code);
   end;
   Result := nil;
end;

function CHMllpServerClientConnectedCallback
(
   Receiver : TCHMllpServer;
   Client : TCHMconnectionHandle
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnClientConnected) then
   begin
      Receiver.FOnClientConnected(Receiver, CHMllpConnectionCreateObjectWithHandle(Client));
   end;
   Result := nil;
end;

function CHMllpServerClientDisconnectedCallback
(
   Receiver : TCHMllpServer;
   Client : TCHMconnectionHandle
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnClientDisconnected) then
   begin
      Receiver.FOnClientDisconnected(Receiver, CHMllpConnectionCreateObjectWithHandle(Client));
   end;
   Result := nil;
end;

function CHMllpServerClientErrorReceivedCallback
(
   Receiver : TCHMllpServer;
   Client : TCHMconnectionHandle;
   ErrorMessage : PChar;
   ErrorCode : Integer
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnClientErrorReceived) then
   begin
      Receiver.FOnClientErrorReceived(Receiver, CHMllpConnectionCreateObjectWithHandle(Client), PChar(ErrorMessage), ErrorCode);
   end;
   Result := nil;
end;

function CHMllpServerMessageReceivedCallback
(
   Receiver : TCHMllpServer;
   Client : TCHMconnectionHandle;
   Message : PChar
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnMessageReceived) then
   begin
      Receiver.FOnMessageReceived(Receiver, CHMllpConnectionCreateObjectWithHandle(Client), PChar(Message));
   end;
   Result := nil;
end;

function CHMllpServerDataIgnoredCallback
(
   Receiver : TCHMllpServer;
   Client : TCHMconnectionHandle;
   Data : PChar
) : TCHMresultHandle; stdcall;
begin
   if Assigned(Receiver.FOnDataIgnored) then
   begin
      Receiver.FOnDataIgnored(Receiver, CHMllpConnectionCreateObjectWithHandle(Client), PChar(Data));
   end;
   Result := nil;
end;

function CHMchameleonEncodingCreate
(
   ReturnValue : TCHMchameleonEncodingHandlePtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingCreate@4';

function CHMchameleonEncodingAddRef
(
   Handle : TCHMchameleonEncodingHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingAddRef@4';

function CHMchameleonEncodingRelease
(
   Handle : TCHMchameleonEncodingHandle
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingRelease@4';

function CHMchameleonEncodingGetCurrentTranslationLocale
(
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingGetCurrentTranslationLocale@4';

function CHMchameleonEncodingSetCurrentTranslationLocale
(NewLocale : PChar
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingSetCurrentTranslationLocale@4';

function CHMchameleonEncodingCountOfTranslationLocale
(
   ReturnValue : TIntegerPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingCountOfTranslationLocale@4';

function CHMchameleonEncodingTranslationLocale
(LocaleIndex : Integer;
   ReturnValue : TPCharPtr
) : TCHMresultHandle; stdcall; 
external 'NETDLL2.dll' name '_CHMchameleonEncodingTranslationLocale@8';

constructor TCHMchameleonEncoding.Create(Owner: TComponent);
begin
   FHandle := nil;
   inherited Create(Owner);
   DLLcheckResult(CHMchameleonEncodingCreate(Addr(FHandle)));
   DLLcheckResult(CHMchameleonEncodingAddRef(FHandle));
end;

destructor TCHMchameleonEncoding.Destroy;
begin
   DLLcheckResult(CHMchameleonEncodingRelease(FHandle));
   inherited Destroy;
end;

class function TCHMchameleonEncoding.GetCurrentTranslationLocale : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMchameleonEncodingGetCurrentTranslationLocale(Addr(ResultValue)));
   Result := ResultValue;
end;

class procedure TCHMchameleonEncoding.SetCurrentTranslationLocale
(
   NewLocale : String
);
begin
   DLLcheckResult(CHMchameleonEncodingSetCurrentTranslationLocale(PChar(NewLocale)));
end;

class function TCHMchameleonEncoding.CountOfTranslationLocale : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(CHMchameleonEncodingCountOfTranslationLocale(Addr(ResultValue)));
   Result := ResultValue;
end;

class function TCHMchameleonEncoding.TranslationLocale
(
   LocaleIndex : Integer
) : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(CHMchameleonEncodingTranslationLocale(LocaleIndex, Addr(ResultValue)));
   Result := ResultValue;
end;

initialization
   
   CHMclientSetConnectCallback(CHMllpClientConnectedCallback);
   CHMclientSetClosedCallback(CHMllpClientClosedCallback);
   CHMclientSetMessageCallback(CHMllpClientMessageReceivedCallback);
   CHMclientSetIgnoredDataCallback(CHMllpClientDataIgnoredCallback);
   CHMclientSetErrorCallback(CHMllpClientErrorReceivedCallback);
   
   CHMlistenerSetListenerErrorCallback(CHMllpServerErrorReceivedCallback);
   CHMlistenerSetConnectCallback(CHMllpServerClientConnectedCallback);
   CHMlistenerSetDisconnectCallback(CHMllpServerClientDisconnectedCallback);
   CHMlistenerSetConnectionErrorCallback(CHMllpServerClientErrorReceivedCallback);
   CHMlistenerSetMessageCallback(CHMllpServerMessageReceivedCallback);
   CHMlistenerSetIgnoredDataCallback(CHMllpServerDataIgnoredCallback);
   
end.
