import com.interfaceware.chameleon.*;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Class: ServerExample
 * 
 * Description:
 * 
 * The ServerExample is an example of handling the server networking events that
 * are dispatched from the operating system.
 */
class ServerExample {

    class ServerLog implements Log {
        /*
         * A real HL7 interface should have some type of system of logging into log
         * files.  Iguana, for instance, has a very nice logging system that allows 
         * administrators to drill into and browse processed messages and errors.  
         * How you implement this logging is up to your requirements for run time 
         * management of your HL7 interface.
         * 
         * For this example, we will log to the Java console.
         */
        public void println(String Line) {
            System.out.println(Line);
        }
    }
    
    class MyServerListener implements LlpServerListener {

        /* 
         * This is the first event we would expect to get from a client connecting
         * to us.  Notice that all the networking events have a 'Client' object 
         * that can be used to interact with the peer machine that is sending HL7 
         * data.
         */
        public void clientConnected(LlpConnection Client) {
            try {
                printSeparator();
                println("Client connected from " + Client.getRemoteIpAddressAsString());
            } catch (Exception e) {
                println(e.toString());
            }   
        }

        /* 
         * This event is fired if we have an error on a given TCP/IP connection. 
         * When you develop a real application you should log this information for
         * troubleshooting purposes.
         */
        public void clientErrorReceived(LlpConnection Client, String ErrorMessage, int ErrorCode) {
            try {
                printSeparator();
                println("Client error from " + Client.getRemoteIpAddressAsString() + ":");
                println(ErrorMessage);
            } catch (Exception e) {
                println(e.toString());
            } 
        }

        /* 
         * This event is fired if we have an error related to the listening socket 
         * itself rather than a specific connection. In practice this event is 
         * rarely invoked.
         */
        public void errorReceived(String ErrorMessage, int Code) {
            printSeparator();
            println("Server error:");
            println(ErrorMessage);
        }

        /* 
         * HL7 data is sent via the Lower Layer Protocol. Read the manual for more
         * detailed background on this protocol. If you have a badly behaved 
         * counter party it is possible they may send data that is outside of the 
         * LLP start of block and end of block characters - this event will tell 
         * you when this problem occurs.
         */
        public void dataIgnored(LlpConnection Client, String Data) {
            try {
                printSeparator();
                println("Ignored data outside of LLP connection from client " + Client.getRemoteIpAddressAsString() + ":");
                println(Data);
            } catch (Exception e) {
                println(e.toString());
            } 
        }

        /* 
         * This event is the main entry point for where we get an actual HL7 
         * message off 'the wire' from a connected peer. Notice that we 'manually'
         * invoke the parseMessage method on the parseEngine object. This will fire
         * one of the generated onMessage events which are defined in the 
         * ServerEngine class.
         * 
         * The LLP TCP/IP transport classes are not tightly coupled to the
         * HL7Engine object. You can use any kind of transport you like with 
         * Chameleon, including your own transports or third party components.
         * 
         * In this application we do not block on events - this is because the
         * networking components are asynchronous. You can write synchronous 
         * applications using Chameleon but you would need to use another 
         * transport mechanism other than the LLP TCP/IP components supplied here.
         */
        public void messageReceived(LlpConnection Client, String Message) {
            String responseMessage;

            try {
                printSeparator();
                println("Message receieved from " + Client.getRemoteIpAddressAsString() + ":");

                /*
                 * The makeMessagePrintable function converts \r characters
                 * (which are usually the segment delimiters of an HL7 message)
                 * into the string given in the second argument of 
                 * MakeMessagePrintable. Since \r (carriage return) only places
                 * the cursor back to the beginning of the console line, we 
                 * replace it with \n (line feed), which appends a new line to
                 * the console.
                 * 
                 * The third argument is the maximum length of the message 
                 * before we truncate it. If logging the message consumes too 
                 * much processing time, we may specify a maximum length to 
                 * prevent the Server application from hanging.
                 */
                println(server.makeMessagePrintable(Message,"\n",-1));

                parseEngine.parseMessage(Message, null);

                /*
                 * After consuming the message, we try to generate an ACK using the
                 * auto-acknowledgement engine.
                 */
                responseMessage = ackEngine.generateAckFromMessage(Message);
            } catch (Exception e) {
                println(e.toString());
                try {
                    /*
                     * If parseMessage and/or generateAckFromMessage fails above,
                     * we try to generate a NACK using the negative-acknowledgement
                     * engine.
                    */
                    responseMessage = nackEngine.generateAckFromMessage(Message);
                } catch (Exception nackException) {
                    /*
                     * The negative-acknowledgement engine failed with this 
                     * message, so we generate a hard-coded NACK message. This 
                     * should only really happen when there is real garbage.
                     */
                    responseMessage = "MSH|^~\\&|SENDING_APP|SENDING_FACILITY|RECEIVING_APP|RECEIVING_FACILITY|||NACK|AXYZ|P|2.5|\rMSA|AA|XYZ|\r";
                }
            }

            try {
                Client.transmitMessage(responseMessage);
            } catch (Exception e) {
                println(e.toString());
            }
        }
    
        /* 
         * This is the event we would expect when a client disconnects. Note that
         * sometimes if the client is not well behaved we may not receive this 
         * event.
         */
        public void clientDisconnected(LlpConnection Client) {
            try {
                printSeparator();
                println("Client disconnected from " + Client.getRemoteIpAddressAsString());
            } catch (Exception e) {
                println(e.toString());
            } 
        }
    }

    LlpServer server;
    ServerEngine parseEngine;
    DynamicEngine ackEngine;
    DynamicEngine nackEngine;

    /**
     * Main entry point.
     */
    static public void main(String[] Args) throws Exception {
        ServerExample Server = new ServerExample();
        Server.startServer();
    }

    /**
     * Constructor.
     */
    public ServerExample() throws Exception {
        server = new LlpServer();
        server.addListener(new MyServerListener());
    }

    public void startServer() {
        try {
            System.out.println("HL7 Server started");
            initializeEngine();

            /* 
             * Specifies the thread that will be run when ConsoleServer 
             * terminates. See the class definition for ShutdownThread below
             * for more details.
             */
            Runtime.getRuntime().addShutdownHook(new ShutdownThread());

            /* 
             * runMessageLoop() is a blocking call to listen for networking
             * events from the operating system, and then dispatch these events
             * to the listenerInstance.
             */ 
            LlpServer.runMessageLoop();
        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }  

    /* 
     * The ShutdownThread will be run when the ConsoleServer 
     * terminates. This thread is guaranteed to run no matter how the
     * ConsoleServer is terminated.
     * 
     * This ensures that when ConsoleServer terminates, we stop listening for
     * networking events from the operating system, close the server listening
     * port, and also close any open client connection sockets. 
     */
    class ShutdownThread extends Thread	{
        public void run() {
            try {
                server.stopMessageLoop();
                server.stopListen();
                server.disconnectAllClients();
            } catch (Exception e) {
                System.out.println(e.toString());
            } finally {
                System.out.println("-----");
                System.out.println("HL7 Server terminated");
            }
        }
    }

    public void println(String Line) {
        System.out.println(Line);
    }

    public void printPrompt(String Prompt) {
        System.out.print(Prompt);
    }

    public void printSeparator() {
        println("-----");
    }

    public void initializeEngine() throws Exception {
        printSeparator();

        /*
         * The first step in making the generated parsing engine component 
         * useable is to load the message definition or 'vmd' file. This file 
         * contains all the definitions for the messages, how they are 
         * identified and how they should be mapped into the generated data 
         * objects based on the tables that you define in the Chameleon GUI.
         *  
         * ServerEngine subclasses from HL7Engine since it implements the 
         * actual handling for each particular type of message. HL7Engine is a
         * generated engine which means that it has an interface based on the 
         * Messages and Tables you define in your vmd file.  The use of stub 
         * code is helpful since it means that the compiler can catch typos or
         * spelling mistakes of fields at compile rather than run time. 
         * However, it is not the only way you can use Chameleon. It is 
         * possible to use a non-stubcode interface for the Chameleon parsing
         * engine which we call the 'dynamic interface'. Please review the 
         * manual documentation for more details if you need this functionality
         * - most customers find that the stub code interface works well for 
         * their applications.
         */
        println("Loading vmd files...");
        parseEngine = new ServerEngine();
	parseEngine.addLog(new ServerLog());
        parseEngine.load("test_data.vmd");
        println("test_data.vmd loaded");

        /* 
         * This example has two other Chameleon HL7 engines to help with the 
         * generation of HL7 ACKnowledgement and Negative ACKnowledgment 
         * messages. In this case each engine object is not a stubcode 
         * generated engine but is a dynamic engine. For more information
         * please read the Chameleon manual about auto-acknowledgement.
         */
        ackEngine = new DynamicEngine();
	ackEngine.addLog(new ServerLog());
        ackEngine.load("autoack.vmd");
        println("autoack.vmd loaded");
        nackEngine = new DynamicEngine();
	ackEngine.addLog(new ServerLog());
        nackEngine.load("autonack.vmd");
        println("autonack.vmd loaded");

        printSeparator();

        /*
         * Prompting for user input is just part of mechanics of this 
         * example which is for teaching purposes only.  A real HL7 server 
         * would most likely be part of a backend service.
         */
        BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
        println("Available configurations in test_data.vmd:");
        for (int ConfigIndex = 0; ConfigIndex < parseEngine.countOfConfig(); ConfigIndex++) {
            println(ConfigIndex + ") " + parseEngine.configName(ConfigIndex));
        }
        printPrompt("Enter configuration index (current: " + parseEngine.currentConfigIndex() + "): ");
        String ConfigInput = stdin.readLine();
        if (ConfigInput.length() > 0) {
            parseEngine.setCurrentConfigIndex(Integer.valueOf(ConfigInput).intValue());
        }
        printSeparator();
        println("Using " + parseEngine.configName(parseEngine.currentConfigIndex()) + " configuration");

        printSeparator();

        int PortNumber = 5123;
        printPrompt("Enter port to listen on (default: " + PortNumber + "): ");
        String PortInput = stdin.readLine();
        if (PortInput.length() > 0) {
            PortNumber = Integer.valueOf(PortInput).intValue();
        }
        server.startListen(PortNumber);
        printSeparator();
        println("Listening on port " + PortNumber);

        printSeparator();

        println("Control + C to terminate the server...");
    }
}
